<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Block\Widget;

use Flat101\MultiWidget\Api\Data\ItemsInterface;
use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Api\MultiWidgetRepositoryInterface;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\Template;
use Magento\Widget\Block\BlockInterface;

abstract class SlickBlock extends Template implements BlockInterface
{
    /**
     * SLICK CONSTANTS
     */
    const DEFAULT_AUTOPLAY = false;
    const DEFAULT_AUTOPLAY_SPEED = 3500;
    const DEFAULT_INFINITE = false;
    const DEFAULT_ROWS = 1;
    const DEFAULT_ITEMS = 1;
    const DEFAULT_SHOW_ARROWS = false;
    const DEFAULT_SHOW_DOTS = false;
    const DEFAULT_FADE = false;
    const DEFAULT_CSS_EASE = "ease";

    const DEFAULT_HEADING = 'div';

    /** @var MultiWidgetRepositoryInterface */
    private MultiWidgetRepositoryInterface $multiWidgetRepository;

    /** @var SearchCriteriaBuilder */
    private SearchCriteriaBuilder $searchCriteriaBuilder;

    /** @var SortOrderBuilder */
    private SortOrderBuilder $sortOrderBuilder;

    /** @var FilterBuilder */
    private FilterBuilder $filterBuilder;

    /**
     * SlickBlock constructor.
     *
     * @param Template\Context $context
     * @param MultiWidgetRepositoryInterface $multiWidgetRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     * @param SortOrderBuilder $sortOrderBuilder
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        MultiWidgetRepositoryInterface $multiWidgetRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder,
        SortOrderBuilder $sortOrderBuilder,
        array $data = []
    ) {
        $this->multiWidgetRepository    = $multiWidgetRepository;
        $this->searchCriteriaBuilder    = $searchCriteriaBuilder;
        $this->filterBuilder            = $filterBuilder;
        $this->sortOrderBuilder         = $sortOrderBuilder;
        parent::__construct($context, $data);
    }

    /**
     * Return if have to autoplay
     *
     * @return bool
     */
    public function hasAutoplay(): bool
    {
        if (!$this->hasData('autoplay')) {
            $this->setData('autoplay', self::DEFAULT_AUTOPLAY);
        }
        return (bool)$this->getData('autoplay');
    }

    /**
     * Return autoplay timeout millis
     *
     * @return int
     */
    public function getAutoplaySpeed(): int
    {
        if (!$this->hasData('autoplay_speed')) {
            $this->setData('autoplay_speed', self::DEFAULT_AUTOPLAY_SPEED);
        }
        return (int)$this->getData('autoplay_speed');
    }

    /**
     * Return if have to infinite loop
     *
     * @return bool
     */
    public function hasInfinite(): bool
    {
        if (!$this->hasData('infinite')) {
            $this->setData('infinite', self::DEFAULT_INFINITE);
        }
        return (bool)$this->getData('infinite');
    }

    /**
     * Return rows
     *
     * @param $totalItems
     * @return int
     */
    public function getRows($totalItems): int
    {
        $itemsToShow = $this->getItemsToShow();

        if ($itemsToShow >= $totalItems) {
            $this->setData('items_to_show', $totalItems);
            $this->setData('rows', self::DEFAULT_ROWS);
        }
        if ($itemsToShow < $totalItems) {
            $rowNumber = intval(ceil($totalItems / $itemsToShow));
            $this->setData('rows', $rowNumber);
        }

        return (int)$this->getData('rows');
    }

    /**
     * Return items to show
     *
     * @return int
     */
    public function getItemsToShow(): int
    {
        if (!$this->hasData('items_to_show')) {
            $this->setData('items_to_show', self::DEFAULT_ITEMS);
        }
        return (int)$this->getData('items_to_show');
    }

    /**
     * Return css ease effect
     *
     * @return string
     */
    public function getCssEase(): string
    {
        if (!$this->hasData('css_ease')) {
            $this->setData('css_ease', self::DEFAULT_CSS_EASE);
        }
        return $this->getData('css_ease');
    }

    /**
     * Return if have to fade
     *
     * @return bool
     */
    public function getFade(): bool
    {
        if (!$this->hasData('fade')) {
            $this->setData('fade', self::DEFAULT_FADE);
        }
        return (bool)$this->getData('fade');
    }

    /**
     * Return if have to shown or not dots
     *
     * @return bool
     */
    public function showDots(): bool
    {
        if (!$this->hasData('show_dots')) {
            $this->setData('show_dots', self::DEFAULT_SHOW_DOTS);
        }
        return (bool)$this->getData('show_dots');
    }

    /**
     * Return if have to shown or not dots
     *
     * @return bool
     */
    public function showArrows(): bool
    {
        if (!$this->hasData('show_arrows')) {
            $this->setData('show_arrows', self::DEFAULT_SHOW_ARROWS);
        }
        return (bool)$this->getData('show_arrows');
    }

    /**
     * Return if have to shown or not title and subtitle's block
     *
     * @return bool
     */
    public function showTitleBlock(): bool
    {
        return (bool)$this->getData('show_title_block');
    }

    /**
     * @throws NoSuchEntityException
     */
    public function getMultiWidget(): MultiWidgetInterface
    {
        $multiWidgetId = intval($this->getData('multiwidget_id'));
        return $this->multiWidgetRepository->getById($multiWidgetId);
    }

    /**
     * @throws NoSuchEntityException
     */
    public function canShowMultiWidget(): bool
    {
        return $this->canShow($this->getMultiWidget());
    }

    /**
     * @throws NoSuchEntityException
     */
    public function canShowItem($item): bool
    {
        return $this->canShow($item);
    }

    /**
     * @param $entity
     * @return bool
     * @throws \Exception
     */
    protected function canShow($entity)
    {
        if (intval($entity->getStatus() == 0)) {
            return false;
        }

        $visibleFrom = $entity->getVisibleFrom();
        $visibleTo = $entity->getVisibleTo();
        if ($visibleFrom || $visibleTo) {
            $today = new \DateTime('now');

            if (!empty($visibleFrom)) {
                $visibleFrom = new \DateTime($visibleFrom);
            }
            if (!empty($visibleTo)) {
                $visibleTo = new \DateTime($visibleTo);
            }

            if ((!empty($visibleFrom)) AND $visibleFrom > $today) {
                return false;
            }

            if ((!empty($visibleTo)) AND $today > $visibleTo) {
                return false;
            }
        }

        return true;
    }


    /**
     * @return \Flat101\MultiWidget\Api\Data\ItemsInterface[]
     */
    public function getItems(): ?array
    {
        try {
            $multiWidgetId = intval($this->getData('multiwidget_id'));
            $items = $this->multiWidgetRepository->getItemsById($multiWidgetId, true);

            return $items->getItems();
        } catch (NoSuchEntityException $e) {
            return null;
        }
    }

    /**
     * @param string $imageName
     * @return string
     */
    abstract public function getImageUrl(string $imageName): string;

    /**
     * @param ItemsInterface $item
     * @return bool
     */
    public function hasMobileImage(ItemsInterface $item): bool
    {
        return (bool)$item->getImageMobile();
    }

    /**
     * @param ItemsInterface $item
     * @return bool
     */
    public function hasTabletImage(ItemsInterface $item): bool
    {
        return (bool)$item->getImageTablet();
    }

    /**
     * @param ItemsInterface $item
     * @return bool
     */
    public function hasTitle(ItemsInterface $item): bool
    {
        return (bool)$item->getTitle();
    }

    /**
     * @param ItemsInterface $item
     * @return bool
     */
    public function hasSubtitle(ItemsInterface $item): bool
    {
        return (bool)$item->getSubtitle();
    }

    /**
     * @param ItemsInterface $item
     * @return bool
     */
    public function hasButton(ItemsInterface $item): bool
    {
        return (bool)($item->getButtonText() && $item->getButtonLink());
    }

    /**
     * @param string $heading
     * @return string
     */
    public function manageHeadings(string $heading): string
    {
        if (strcmp($heading, 'none') === 0) {
            return self::DEFAULT_HEADING;
        }

        return $heading;
    }
}
