<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Controller\Adminhtml\AbstractController;

use Flat101\MultiWidget\Api\Data\MultiWidgetInterface;
use Flat101\MultiWidget\Api\ItemsRepositoryInterface;
use Flat101\MultiWidget\Api\MultiWidgetRepositoryInterface;
use Flat101\MultiWidget\Api\Data\ItemsInterface;
use Flat101\MultiWidget\Helper\FileEraser;
use Flat101\MultiWidget\Model\ItemsFactory;
use Flat101\MultiWidget\Model\MultiWidgetFactory;
use Flat101\MultiWidget\Model\ResourceModel\Items\Collection;
use Flat101\MultiWidget\Model\ResourceModel\Items\CollectionFactory;
use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\Redirect;
use Magento\Backend\App\Action;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Abstract class for controllers Save(All MultiWidget Entities)
 */
abstract class Save extends Action implements HttpPostActionInterface
{
    /** @var MultiWidgetFactory */
    protected MultiWidgetFactory $multiWidgetFactory;

    /** @var MultiWidgetRepositoryInterface */
    protected MultiWidgetRepositoryInterface $multiWidgetRepository;

    /** @var ItemsFactory */
    protected ItemsFactory $itemsFactory;

    /** @var ItemsRepositoryInterface */
    protected ItemsRepositoryInterface $itemsRepository;

    /** @var CollectionFactory */
    protected CollectionFactory $itemsCollectionFactory;

    /** @var FileEraser */
    protected FileEraser $fileEraser;

    /**
     * @param Context $context
     * @param MultiWidgetFactory $multiWidgetFactory
     * @param MultiWidgetRepositoryInterface $multiWidgetRepository
     * @param ItemsFactory $itemsFactory
     * @param ItemsRepositoryInterface $itemsRepository
     * @param CollectionFactory $itemsCollectionFactory
     * @param FileEraser $fileEraser
     */
    public function __construct(
        Context $context,
        MultiWidgetFactory $multiWidgetFactory,
        MultiWidgetRepositoryInterface $multiWidgetRepository,
        ItemsFactory $itemsFactory,
        ItemsRepositoryInterface $itemsRepository,
        CollectionFactory $itemsCollectionFactory,
        FileEraser $fileEraser
    ) {
        $this->multiWidgetFactory = $multiWidgetFactory;
        $this->multiWidgetRepository = $multiWidgetRepository;
        $this->itemsFactory = $itemsFactory;
        $this->itemsRepository = $itemsRepository;
        $this->itemsCollectionFactory = $itemsCollectionFactory;
        $this->fileEraser = $fileEraser;
        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return ResultInterface
     */
    public function execute(): ResultInterface
    {
        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        $data = $this->getRequest()->getPostValue();
        $multiWidget = $this->multiWidgetFactory->create();

        if ($data) {
            if (empty($data['entity_id'])) {
                $data['entity_id'] = null;
            }
            $data['status'] = intval($data['status']);
            $data['type_id'] = intval($data['type_id']);

            $id = intval($this->getRequest()->getParam('entity_id'));
            if ($id) {
                try {
                    $multiWidget = $this->multiWidgetRepository->getById($id);
                } catch (LocalizedException $e) {
                    $this->messageManager->addErrorMessage(__('This slider no longer exists.'));
                    return $resultRedirect->setPath('*/*/');
                }
            }

            $multiWidget->setData($data);

            try {
                $this->multiWidgetRepository->save($multiWidget);
                $this->manageItems($multiWidget, $data['items']);
                $this->messageManager->addSuccessMessage('Entity was saved successfully.');
                return $this->processBlockReturn($multiWidget, $data, $resultRedirect);
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving entity.'));
            }

            return $resultRedirect->setPath('*/*/edit', ['entity_id' => $id]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    /**
     * Manage Items of MultiWidget
     *
     * @param $multiWidget
     * @param $rowItems
     */
    protected function manageItems($multiWidget, $rowItems)
    {
        try {
            $itemsIds = [];
            if ($rowItems && count($rowItems)) {
                foreach ($rowItems as $row) {
                    if (!empty($row[ItemsInterface::ID])) {
                        try {
                            $item = $this->itemsRepository->getById(intval($row[ItemsInterface::ID]));
                        } catch (NoSuchEntityException $e) {
                            $item = $this->itemsFactory->create();
                        }
                    } else {
                        $item = $this->itemsFactory->create();
                    }

                    $row[ItemsInterface::STATUS] = intval($row[ItemsInterface::STATUS]);
                    $row[ItemsInterface::IMAGE] = $row[ItemsInterface::IMAGE][0]['name'];
                    if (!empty($row[ItemsInterface::IMAGE_TABLET])) {
                        $row[ItemsInterface::IMAGE_TABLET] = $row[ItemsInterface::IMAGE_TABLET][0]['name'];
                    }
                    if (!empty($row[ItemsInterface::IMAGE_MOBILE])) {
                        $row[ItemsInterface::IMAGE_MOBILE] = $row[ItemsInterface::IMAGE_MOBILE][0]['name'];
                    }
                    $row[ItemsInterface::POSITION] = intval($row[ItemsInterface::POSITION]);
                    if (empty($row[ItemsInterface::MULTI_WIDGET_ID])) {
                        $row[ItemsInterface::MULTI_WIDGET_ID] = intval($multiWidget->getId());
                    }

                    $item->setData($row);

                    try {
                        $this->itemsRepository->save($item);
                        $itemsIds[] = $item->getId();
                    } catch (AlreadyExistsException $e) {
                        $this->messageManager->addErrorMessage('There was an error saving entity\'s items: ' . $e->getMessage());
                    }
                }
            }

            $this->deleteRemovedItems($multiWidget, $itemsIds);
        } catch (AlreadyExistsException $e) {
            $this->messageManager->addErrorMessage('There was an error saving entity: ' . $e->getMessage());
        }
    }

    /**
     * Remove Items that was deleted by user
     *
     * @param MultiWidgetInterface $multiWidget
     * @param array $itemsIds
     */
    abstract public function deleteRemovedItems(MultiWidgetInterface $multiWidget, array $itemsIds);

    /**
     * Process and set the block return
     *
     * @param MultiWidgetInterface $multiWidget
     * @param array $data
     * @param ResultInterface $resultRedirect
     * @return ResultInterface
     */
    protected function processBlockReturn(
        MultiWidgetInterface $multiWidget,
        array $data,
        ResultInterface $resultRedirect
    ): ResultInterface
    {
        $redirect = $data['back'] ?? 'close';

        if ($redirect ==='edit') {
            $resultRedirect->setPath('*/*/edit', ['entity_id' => $multiWidget->getId()]);
        }

        if ($redirect === 'close') {
            $resultRedirect->setPath('*/*/');
        }

        return $resultRedirect;
    }

    /**
     * @param MultiWidgetInterface $multiWidget
     * @return Collection
     */
    protected function getItemsCollection(MultiWidgetInterface $multiWidget): Collection
    {
        $itemsCollection = $this->itemsCollectionFactory->create();
        $itemsCollection
            ->addFieldToFilter(ItemsInterface::MULTI_WIDGET_ID, $multiWidget->getId())
            ->setOrder(ItemsInterface::POSITION, 'ASC');

        return $itemsCollection;
    }
}
