<?php
declare(strict_types=1);

namespace Flat101\MultiWidget\Model;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Flat101\MultiWidget\Api\Data\ItemsSearchResultsInterface;
use Flat101\MultiWidget\Api\Data\ItemsSearchResultsInterfaceFactory;
use Flat101\MultiWidget\Model\ResourceModel\Items\Collection as ItemsCollection;
use Flat101\MultiWidget\Model\ResourceModel\Items\CollectionFactory as ItemsCollectionFactory;
use Flat101\MultiWidget\Api\Data\ItemsInterface;
use Flat101\MultiWidget\Api\ItemsRepositoryInterface;
use Flat101\MultiWidget\Model\ResourceModel\Items as ResourceModelItems;
use Flat101\MultiWidget\Model\Items;
use Flat101\MultiWidget\Model\ItemsFactory;
use \Magento\Framework\Exception\AlreadyExistsException;

class ItemsRepository implements ItemsRepositoryInterface
{
    /** @var ItemsFactory */
    private $itemsFactory;

    /** @var ResourceModelItems */
    private $resourceModelItems;

    /** @var ItemsCollectionFactory */
    private $itemsCollectionFactory;

    /** @var ItemsSearchResultsInterfaceFactory */
    private $itemsSearchResultsInterfaceFactory;

    /** @var CollectionProcessorInterface */
    private $collectionProcessor;

    /** @var JoinProcessorInterface */
    private $extensionAttributesJoinProcessor;

    /**
     * TypesRepository constructor.
     *
     * @param ItemsFactory $itemsFactory
     * @param ResourceModelItems $resourceModelItems
     * @param ItemsCollectionFactory $itemsCollectionFactory
     * @param ItemsSearchResultsInterfaceFactory $itemsSearchResultsInterfaceFactory
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     */
    public function __construct(
        ItemsFactory $itemsFactory,
        ResourceModelItems $resourceModelItems,
        ItemsCollectionFactory $itemsCollectionFactory,
        ItemsSearchResultsInterfaceFactory $itemsSearchResultsInterfaceFactory,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor
    ) {
        $this->itemsFactory                       = $itemsFactory;
        $this->resourceModelItems                 = $resourceModelItems;
        $this->itemsCollectionFactory             = $itemsCollectionFactory;
        $this->itemsSearchResultsInterfaceFactory = $itemsSearchResultsInterfaceFactory;
        $this->collectionProcessor                = $collectionProcessor;
        $this->extensionAttributesJoinProcessor   = $extensionAttributesJoinProcessor;
    }

    /**
     * @inheritdoc
     * @throws AlreadyExistsException
     */
    public function save(ItemsInterface $items): ItemsInterface
    {
        $this->resourceModelItems->save($items);

        return $items;
    }

    /**
     * @inheritdoc
     */
    public function getById(int $entityId): ItemsInterface
    {
        return $this->get($entityId);
    }

    /**
     * @inheritdoc
     */
    public function get($value, ?string $attributeCode = null): ItemsInterface
    {
        $items = $this->itemsFactory->create()->load($value, $attributeCode);

        if (!$items->getId()) {
            throw new NoSuchEntityException(__('Unable to find Multi Widget Item'));
        }

        return $items;
    }

    /**
     * @inheritdoc
     */
    public function delete(ItemsInterface $item): bool
    {
        $entityId = $item->getId();
        try {
            $this->resourceModelItems->delete($item);
        } catch (\Exception $e) {
            throw new CouldNotDeleteException(
                __('Unable to remove Multi Widget Item %1', $entityId)
            );
        }

        return true;
    }

    /**
     * @inheritdoc
     */
    public function deleteById(int $entityId): bool
    {
        $entity = $this->getById($entityId);

        return $this->delete($entity);
    }

    /**
     * @inheritdoc
     */
    public function getList(SearchCriteriaInterface $searchCriteria): ItemsSearchResultsInterface
    {
        /** @var ItemsCollection $collection */
        $collection = $this->itemsCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process($collection, ItemsInterface::class);
        $this->collectionProcessor->process($searchCriteria, $collection);

        /** @var ItemsSearchResultsInterface $searchResults */
        $searchResults = $this->itemsSearchResultsInterfaceFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getItems());
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }
}
